#!/bin/sh
CONFFILE='/etc/asterisk/lcdialsh/LCDial.sh.conf'
ENUMCONFFILE='/etc/asterisk/enum.conf'
#
# This version, if the "dig" executable is in the path, 
# replaces the call to the Asterisk app EnumLookup
# with an invocation to "dig -t naptr" to resolve ALL the
# NAPTR ENUM records for SIP and IAX URI's, and treats them as
# zero-cost providers, trying the respective URI's 
# sequentially.
# dig for OpenWRT can be found in the "bind-utils" package:
# http://tracker.openwrt.org/search/?b=search&query=bind-utils
# As the package is pretty large, just extract dig with tar
# or 7zip and place it manually in /usr/bin
# Shortcomings:
# - order and preference are still disregarded in this version
# - "tel:" URI's are ignored, whereas according to RFC2916
#   they should restart the resolution process with the new
#   number, unless this is equal to the original (what about RFC3761?)
#
# Copyright (C) Enzo Michelangeli 2005
#
# --------- no user-configurable parameters below this line
#

. /usr/lib/asterisk/agi-bin/sh-agi.inc

lf="
"
tab=$(echo '_' | tr '_' '\t')  # sigh...

do_dial() {
	sh_agi_cmd EXEC Dial \""$1"\"
	# sh_agi_log "After Dial: sh_agi_response=$sh_agi_response, sh_agi_result=$sh_agi_result"
	sh_agi_getvariable DIALSTATUS
	sh_agi_log "After Dial: DIALSTATUS=$sh_agi_DIALSTATUS"
	case "$sh_agi_DIALSTATUS" in 
	CONGESTION|CHANUNAVAIL)
	;;
	ANSWER|NOANSWER|BUSY|CANCEL)
		exit 0;
	;;
	*)
		sh_agi_log "After Dial: *** unexpected DIALSTATUS=$sh_agi_DIALSTATUS" "1"
		exit 0;	# unexpected
	;;
	esac
}

# "open_r n xyz.dat" opens for read xyz.dat as FD n
open_r() {      # fd filespec
        local cmd
        cmd="exec $1<$2"
        eval $cmd
}

# close_r n closes the file associated to the FD n and frees the latter
close_r() {     # fd
        local cmd
        cmd="exec $1<&-"
        eval $cmd
}

# find cost per minute in the specified ratefile for the best match of
# the global variable $callednumber. Returns the relative cost in
# millicents per minute into the global variable $cost
#
find_bestmatch_cost() {		# ratefile
	local saveIFS="$IFS"
	IFS=" $lf$tab\","	# For CSV files w/o embedded spaces. NOTE: it must start with ' ' (see ash docs)
	local bestmatch=""
	local prefix
	local prefixcost
	
	local ratefile=$1
	if [ -e "$ratefile" ]; then
		# look for records where the prefix is a substring of $callednumber
		# and formats them into the form "prefix,cost"
		# then sort the latter and pick the last, which is the longest
		# (also because ',' lexicographically precedes [0-9]...
		# sed hackery: kids, don't try this at home!
		local prefcost="$(sed -e "s/\$/_$callednumber/; s/^[^-]*-[^0-9-]\+\([0-9]\+\).*/-,\1/; s/^[^0-9]*\([0-9]\+\)\([^0-9]\+\)\([0-9]\+\)[^0-9]\+\1[0-9]*/\1,\3/;/_/d"  \
			"$ratefile" | sort | tail -1 )"
		case "_$prefcost" in
		_)
			unset cost
		;;
		*)
			cost=$(echo "$prefcost" | cut -d "," -f 2)
		;;
		esac
	else
		sh_agi_log "*** Ratefile $ratefile does not exist!" "1"
	fi

#	case _prefix in
#	_)
#		sh_agi_log "'>>>>>'for $callednumber $ratefile has NO routes"
#	;;
#	*)
#		sh_agi_log "'>>>>>'for $callednumber $ratefile has prefix $prefix at $cost millicents/minute"
#	;;
#	esac
	close_r 4
	IFS="$saveIFS"
}


# $1 is set to the Dial called number (guaranteed to contain only dec. digits)
# $2 is set to the Dial timeout (or unset)
# $3 is set to the Dial options string (or unset)
# $4 is set to the Dial URL string (or unset)
# $5 is set to the max cost in millicents (or unset)

callednumber=$(echo "$1" | sed -e 's/[^0-9]//g') # remove non-digits
case _"$5" in 
_) 
	costlimit=10000000
;;
*)
	costlimit="$(expr $5 \+ 1)"
;;
esac

# Try all the VoIP termination providers serving that number, starting
# with the cheapest for it, and dial only if the cost per minute
# is less or equal to $5.
# ENUM is an "implicit" free provider, and the existence of service
# plus the URI rule is given by listing the NAPTR records for the
# reversed number

local saveIFS="$IFS"
IFS=" $lf$tab"	# A sane value. NOTE: it must start with ' ' (see ash docs)
local line=0
local par1
local par2
local par3
local confdir
local cost
providers_list=""

if dig=$(which dig 2>/dev/null); then 
	open_r 3 "$ENUMCONFFILE"
	while read -r line <&3 ; do
		case "_$line" in
		_)
			continue
		;;
		esac
		ENUMDOM=$(echo "$line" \
		| egrep '[sS][eE][aA][rR][cC][hH][ ]*=>[ ]*' \
		| sed 's/[ ]*[sS][eE][aA][rR][cC][hH][ ]*=>[ ]*\([^ ]\+\)/\1/')
		case "_$ENUMDOM" in
		_)
			continue
		;;
		esac
		#for ENUMDOM in "e164.org" ; do
		# start with an ENUM lookup for all the domains in /etc/asterisk/enum.conf
		CNT=0
		# convert "12345" into "5.4.3.2.1."
		REVNUM=$(echo "$callednumber" \
		| sed '/\n/!G;s/\(.\)\(.*\n\)/&\2\1./;/..*\n/D;s/.//')
		# get the NAPTR records (if any) discarding URI rules not for SIP or IAX(2)
		HOSTOUTPUT=$($dig -t naptr "${REVNUM}${ENUMDOM}." \
		| egrep '"*u"*[ ]+"*E2U\+' \
		| sed 's/.*NAPTR[^0-9]*\(.*\)/\1/' \
		| sed 's/\([0-9]\+\)[^0-9]\+\([0-9]\+\)[ ]\+[^ ]\+[ ]\+[^ ]\+[ ]\+\([^ ]\+\)/\1_\2%\3/' \
		| sed '/\(sip:\|iax:\|iax2:\)/!d;s/\\\\+//;s/(/\\(/;s/)/\\)/' \
		| sed 's/\\\\/\\/;s/"\(.*\)".*/s\1/;s/\^//g;s/\$//g')
				
		for URIrule in $HOSTOUTPUT; do
			ORDER_PREF=$(echo "$URIrule" | cut -d '%' -f 1)
			URIrule=$(echo "$URIrule" | cut -d '%' -f 2)
			URI="$(echo "$callednumber" | sed -e "$URIrule")"
			sh_agi_log "=== found ENUM route with ${ENUMDOM}: $URI"
			# convert from IETF URI to Asterisk URI (sip/domain/number)
			URI="$(echo "$URI" \
			| sed -e 's!\([^:]\+\):\([^@]*\)\@\(.\+\)!\1/\3/\2!')"
			# create a dummy provider domain_priority_counter
			provider=$(echo "${ENUMDOM}_${ORDER_PREF}_${CNT}" | sed -e 's/\./_/g')
			# and place it in a ENUMnn_record with cost=0

			eval "${provider}_URI"=\"$URI\"
			eval "${provider}_cost"=\"0\"
			eval "${provider}_record"=\""0 $provider"\"
			#append ENUMnn to providers_list 
			providers_list="$providers_list $provider"
			CNT=$(($CNT + 1))
		done
	done
	close_r 3
else
	# dig is not avalable, so we can't use it to get
	# all the NAPTR records for SIP and IAX rules. So,
	# let's fall back to the less powerful EnumLookup Asterisk app
	# and try to dial the single URI returned by it:

	sh_agi_cmd EXEC EnumLookup "$callednumber"
	#sh_agi_log "response: $sh_agi_response result: $sh_agi_result"

	sh_agi_getvariable ENUM
	# if set, the ENUM query was successful. The sh_agi_ENUM variable
	# will then be set to something like:
	# IAX2/guest@em.no-ip.com/8829900152401

	sh_agi_log "ENUM var: $sh_agi_ENUM"

	case "$sh_agi_ENUM" in
	IAX* | SIP*) 
		do_dial "$sh_agi_ENUM|$2|$3|$4"
		case "$sh_agi_result" in
		0*)
			return 0
		;;
		esac
		;;
	esac
fi
# now scan the actual providers referenced by $CONFFILE

confline=0
confdir=$(echo "$CONFFILE"|sed -e "s!$(basename $CONFFILE)\$!!") # no dirname in OpenWRT :-(
open_r 3 "$CONFFILE"
while read -r provider ratefile URIrule <&3 ; do
	confline=$(expr $confline + 1)
	case _$provider in
	_\;*|_)
		continue	# comment or empty
	;;
	esac
	# sh_agi_log "confline:$confline: provider:$provider: ratefile:$ratefile: URIrule:$URIrule:" "" 
	case _"$URIrule" in
	_)
		sh_agi_log "*** $CONFFILE:$confline: malformed line (less than three parameters): ignored" "1"
		continue;
	;;
	esac

	eval local provider_defined=\$${provider}_defined
	case _$provider_defined in
	_)
		eval ${provider}_defined=yes
		cost=
		find_bestmatch_cost "$confdir$ratefile" # <<<<<<<<<<<<<<<<<<
		case "_$cost" in
		_)
		;;
		*)
			if expr >/dev/null "$cost" \< "$costlimit"; then 
				sh_agi_log "=== found rate ($cost) < limit $costlimit with $provider"
				URI="$(echo $callednumber | sed -e $URIrule)"

				# store temporarily URIs and (provider, cost) pairs in 
				# kind of associative arrays
				eval "${provider}_URI"=\"$URI\"
				eval "${provider}_cost"=\"$cost\"
				eval "${provider}_record"=\""$cost $provider"\"
				providers_list="$providers_list $provider"
			fi
		;;
		esac
	;;
	*)
		sh_agi_log "*** $CONFFILE:$confline: redefinition of provider $par1: ignored" "1"
	;;
	esac
done
close_r 3
IFS="$saveIFS"

case "_$providers_list" in
_)
	sh_agi_log "**** Sorry! No provider can deliver this call ****"
;;
*)
	IFS=" $tab"	# keep \n as literal here!
	sorted_providers_list=$( (for provider in $providers_list ; do
		eval echo "\$${provider}_record"
		done
	) | sort -n | while read cost provider ; do
		echo $provider 
	done )

	IFS=" $tab$cr$lf"	# treat \n again as separator expanding $sorted_providers_list
	# now providers list is sorted on ascending cost	
	for provider in $sorted_providers_list ; do
		eval URI=\""\$${provider}_URI"\"
		eval cost=\""\$${provider}_cost"\"
		
		sh_agi_log "==\> Dial command: $URI|$2|$3|$4 - cost: $cost millicents/min"
		# if successful, do_dial does not return.
		do_dial "$URI|$2|$3|$4"
	done	
	IFS="$saveIFS"
;;
esac

sh_agi_cmd HANGUP
